import React, { Component } from 'react';
import * as PropTypes from 'prop-types';
import { AgGridReact, AgGridColumn } from 'ag-grid-react';
import DataTableCell from '../DataTableCell/DataTableCell';
import DataTableColumnHeader from '../DataTableColumnHeader/DataTableColumnHeader';
import DataTableFloatingFilter, { FILTER_TYPES, FILTER_OPERATORS, SORT_DIRECTIONS, FILTER_OPTIONS, STRING_FILTER_OPTIONS, NUMERIC_FILTER_OPTIONS } from '../DataTableFloatingFilter/DataTableFloatingFilter';

import 'ag-grid-community/dist/styles/ag-grid.css';
import 'ag-grid-community/dist/styles/ag-theme-material.css';
import './DataTable.css';
import getStyles from './DataTable.styles';

const THEME = 'material';

// @TODO Need a right click handler
// @TODO Need paging props

class DataTable extends Component {
    constructor(props) {
        super(props);

        /**
         * Init hack to force re-render of ag-grid-react component when props change
         * @see https://github.com/ag-grid/ag-grid-react/issues/77#issuecomment-314717545
         */
        this.key = 0;
    }

    render() {
        // this.key += 1;
        const {
            cellRenderer,
            cols,
            contextStyles,
            filterDefs,
            onApplyFilters,
            onClearFilter,
            onClearAllFilters,
            onSortRequested,
            floatingFilterComponent,
            multiSelectMode,
            onChangeFilter,
            onColumnPinned,
            onFilterRequested,
            rows,
            showFilters,
            testID,
        } = this.props;

        console.log('DATA TABLE: RENDER: ', this.props); // eslint-disable-line

        const styles = getStyles(contextStyles);

        // Generate container props
        const containerProps = {
            style: styles.container,
            className: 'c-data-table__container',
        };
        if (testID) { containerProps['data-test-id'] = `${testID}__data-table__container`; }

        // Generate viewport props
        const viewportProps = {
            style: styles.viewport,
            className: `ag-theme-${THEME} c-data-table__viewport`,
        };
        if (testID) { containerProps['data-test-id'] = `${testID}__data-table__viewport`; }

        // Generate data table props
        const dataTableProps = {
            /**
             * Enable React 16 Portals to dynamically create components
             * @see https://www.ag-grid.com/react-more-details/#react-16
             */
            reactNext: true,

            /** The height (in px) of the floating filter row */
            floatingFiltersHeight: 155,

            /** Include ag-grid-react hack */
            key: this.key,

            /** Force all columns to render initially instead of dynamically when scrolling */
            suppressColumnVirtualisation: true,

            /** Set number of rows initially rendered beyond the viewport before dynamically rendering more when scrolling */
            rowBuffer: 10,

            /** Column definitions */
            columnDefs: cols,

            /** Default column definitions */
            defaultColDef: {
                cols,
                cellRendererFramework: cellRenderer,
                columnStyles: styles.column,

                /** Enable/disable showing column filters */
                filter: showFilters,
                floatingFilterComponentFramework: floatingFilterComponent,
                floatingFilterComponentParams: {
                    filterDefs,
                    onApplyFilters,
                    onChangeFilter,
                    onClearFilter,
                    onClearAllFilters,
                    onSortRequested,
                    suppressFilterButton: true,
                },
                headerComponentFramework: DataTableColumnHeader,
                headerComponentParams: {
                    onFilterRequested: (field, showFilter) => {
                        console.log('DATA TABLE: FILTER TOGGLE: FIELD: ', field); // eslint-disable-line
                        console.log('DATA TABLE: FILTER TOGGLE: SHOW FILTER: ', showFilter); // eslint-disable-line
                        if (onFilterRequested) {
                            onFilterRequested(field, showFilter);
                        }
                    },
                    onColumnPinned: (field, pinColumn) => {
                        console.log('DATA TABLE: PIN CHANGED: FIELD: ', field); // eslint-disable-line
                        console.log('DATA TABLE: PIN CHANGED: PIN: ', pinColumn); // eslint-disable-line
                        if (onColumnPinned) {
                            onColumnPinned(field, pinColumn);
                        }
                    },

                    /** Enable/disable showing column filters */
                    showFilters,
                },

                /** Prevent default column menu from displaying */
                suppressMenu: true,

                /** Default to resizable columns */
                resizable: true,
            },

            /** Display floating filter */
            floatingFilter: showFilters,

            /** Called with the ag-grid API after grid is 'ready' */
            onGridReady: this.handleGridReady,

            /** Called on row single-click */
            onRowClicked: this.handleRowClicked,

            /** Called on row double-click */
            onRowDoubleClicked: this.handleRowDoubleClicked,

            /** Array of row properties */
            rowData: rows,

            /** Set multiple row selection mode */
            rowSelection: multiSelectMode ? 'multiple' : 'single',

            /** Prevent individual cell selection in favor of row selection */
            suppressCellSelection: true,
        };

        console.log('DATA TABLE: RENDER: TABLE PROPS: ', dataTableProps); // eslint-disable-line

        return (
            <div { ...containerProps }>
                <div { ...viewportProps }>
                    <AgGridReact { ...dataTableProps }>
                        { this.renderColumns(cols) }
                    </AgGridReact>
                </div>
            </div>
        );
    }

    componentDidUpdate(prevProps) {
        const { showFilters } = this.props;
        if (prevProps.showFilters !== showFilters) {
            this.key += 1;
            this.forceUpdate();
        }
    }

    /**
     * Renders columns
     * @param {Object} columnList - meta data about columns
     * @returns {Array}
     */
    renderColumns = (columnList) => (
        columnList.map((column) => (
            <AgGridColumn
                autoHeight
                field={ column.field }
                filter
                headerName={ column.headerName }
                key={ column.field }
                pinned={ column.pinned }
                resizable
                width={ column.width } />
        ))
    );

    /** Makes grid api available to component */
    handleGridReady = (grid) => {
        this.api = grid.api;
    };

    /**
     * Called on click of a row
     * @returns {void}
     */
    handleRowClicked = () => {
        const { onRowClick } = this.props;

        if (onRowClick) {
            onRowClick({
                data: this.api.getSelectedRows(),
                nodes: this.api.getSelectedNodes(),
            });
        }
    };

    /**
     * Called on double click of a row
     * @returns {void}
     */
    handleRowDoubleClicked = () => {
        const { onRowDoubleClick } = this.props;

        if (onRowDoubleClick) {
            onRowDoubleClick({
                data: this.api.getSelectedRows()[0],
                node: this.api.getSelectedNodes()[0],
            });
        }
    };
}

DataTable.propTypes = {
    /**
     * Custom cell renderer
     * @param {Object} params column details for the cell
     */
    cellRenderer: PropTypes.func,

    /** Array containing column definition objects */
    cols: PropTypes.arrayOf(PropTypes.shape({
        /** Unique column id which maps to row property */
        field: PropTypes.string,

        /** Column header display text */
        headerName: PropTypes.string,

        /** Enables column to be pinned */
        pinned: PropTypes.bool,

        /** Width of column */
        width: PropTypes.number,
    })).isRequired,

    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Column level styles */
        column: PropTypes.object,

        /** Style object for container */
        container: PropTypes.object,

        /** Style object for evenRows*/
        evenRows: PropTypes.object,

        /** Style object for grid filter dialog */
        filter: PropTypes.object,

        /** Style object for grid filter dialog container */
        filterContainer: PropTypes.object,

        /** Style object for grid */
        grid: PropTypes.object,

        /** Style object for header cell */
        headerCell: PropTypes.object,

        /** Style object for header text */
        headerText: PropTypes.object,

        /** Style object for oddRows*/
        oddRows: PropTypes.object,

        /** Style object for grid row */
        row: PropTypes.object,

        /** Style object for selected grid row */
        rowSelected: PropTypes.object,
    }),

    /**
     * Array containing filter definition objects
     * @typedef {Object} FilterDef
     * @property {String} field
     * @property {String} operator
     * @property {String} sortDirection
     * @property {String} type
     * @property {Boolean|Number|String|Date} value
     */
    filterDefs: PropTypes.arrayOf(PropTypes.shape({
        /** Unique column id which maps to row property */
        field: PropTypes.string,

        /** Filter field operator value */
        operator: PropTypes.oneOf([
            FILTER_OPERATORS.NONE,
            FILTER_OPERATORS.EQUAL,
            FILTER_OPERATORS.NOT_EQUAL,
            FILTER_OPERATORS.GREATER_THAN,
            FILTER_OPERATORS.LESS_THAN,
            FILTER_OPERATORS.CONTAINS,
            FILTER_OPERATORS.STARTS_WITH,
            FILTER_OPERATORS.ENDS_WITH,
        ]),

        /** Sort direction */
        sortDirection: PropTypes.oneOf([
            SORT_DIRECTIONS.NONE,
            SORT_DIRECTIONS.ASC,
            SORT_DIRECTIONS.DESC,
        ]),

        /** Sort priority */
        sortPriority: PropTypes.number,

        /** Filter field input type */
        type: PropTypes.oneOf([
            FILTER_TYPES.BOOLEAN,
            FILTER_TYPES.DATE,
            FILTER_TYPES.DATETIME,
            FILTER_TYPES.DECIMAL,
            FILTER_TYPES.INTEGER,
            FILTER_TYPES.STRING,
            FILTER_TYPES.TIME,
            FILTER_TYPES.YEAR,
        ]),

        /** Filter field input value */
        value: PropTypes.oneOfType([
            PropTypes.bool,
            PropTypes.number,
            PropTypes.string,
            PropTypes.instanceOf(Date),
        ]),
    })),

    /** Custom floating filter component */
    floatingFilterComponent: PropTypes.func,

    /** Enables multiple row selection */
    multiSelectMode: PropTypes.bool,

    // TODO: implement to call server for data refresh
    /**
     * Called on click of apply filters button
     * @param {Object} event
     * @param {Object} props
     * @param {String} field
     * @param {Object} filters
     */
    onApplyFilters: PropTypes.func,

    /**
     * Called after change of a filter value
     * @param {Object} event
     * @param {Object} props
     * @param {String} field
     * @param {FilterDef} filterValue
     */
    onChangeFilter: PropTypes.func,

    /**
     * Called on click of clear all filters button
     * @param {Object} event
     * @param {Object} props
     * @param {String} field
     * @param {Object} filterValues
     */
    onClearAllFilters: PropTypes.func,

    /**
     * Called on click of clear single filter button
     * @param {Object} event
     * @param {Object} props
     * @param {String} field
     * @param {Object} filters
     */
    onClearFilter: PropTypes.func,

    // TODO: implement to save column position after data refresh
    /** Called after moving of columns */
    // onColumnMoved: PropTypes.func,

    // TODO: implement to save column pinned state after data refresh
    /**
     * Called after pinning of columns
     * @param {String} field
     * @param {Boolean} pin
     */
    onColumnPinned: PropTypes.func,

    // TODO: implement to save column resized state after data refresh
    /** Called after resize of columns */
    // onColumnResized: PropTypes.func,

    /**
     * Called on filter button click
     * @param {String} field
     * @param {Boolean} showFilter
     */
    onFilterRequested: PropTypes.func,

    /**
     * Called on row click
     * @param { { data: Array<Object>, nodes: Array<Object> } }
     */
    onRowClick: PropTypes.func,

    /**
     * Called on row double click
     * @param { { data: Object, node: Object } }
     */
    onRowDoubleClick: PropTypes.func,

    /**
     * Called after change of a filter value
     * @param {Object} event
     * @param {Object} props
     * @param {String} field
     * @param {String} sortOrder
     */
    onSortRequested: PropTypes.func,

    /** Array of objects of column fields */
    rows: PropTypes.arrayOf(PropTypes.object).isRequired,

    /** Enable / disable visibility of the floating filter (if filterableColumns is enabled) */
    showFilters: PropTypes.bool,

    /** ID used for testing */
    testID: PropTypes.string,
};

DataTable.defaultProps = {
    cellRenderer: DataTableCell,
    floatingFilterComponent: DataTableFloatingFilter,
};

export default DataTable;
export {
    FILTER_TYPES,
    FILTER_OPERATORS,
    SORT_DIRECTIONS,
    FILTER_OPTIONS,
    STRING_FILTER_OPTIONS,
    NUMERIC_FILTER_OPTIONS,
};
