import React, { Fragment, useState, useEffect } from 'react';
import * as PropTypes from 'prop-types';
import {

    // DataTypeProvider, // used to enable custom filter drop down operators
    FilteringState,
    SelectionState,
    VirtualTableState,
} from '@devexpress/dx-react-grid';
import {
    DragDropProvider,
    Grid,
    TableColumnReordering,
    TableColumnResizing,
    TableFilterRow,
    TableFixedColumns,
    TableHeaderRow,
    TableSelection,
    VirtualTable,
} from '@devexpress/dx-react-grid-material-ui';

import ActivityIndicator from '../ActivityIndicator/ActivityIndicator';
import IconButton, { ICON_SIZE } from '../IconButton/IconButton';

import getStyles from './DataTableDXG.styles';

const DataTableCell = VirtualTable.Cell;
const DEFAULT_COLUMN_WIDTH = 150;

/**
 * Enum for filter operations
 * @typedef {Object} FilterOperations
 * @enum {String}
 */
const FILTER_OPERATIONS = {
    contains: 'contains',
    notContains: 'notContains',
    startsWith: 'startsWith',
    endsWith: 'endsWith',
    equal: 'equal',
    notEqual: 'notEqual',
    greaterThan: 'greaterThan',
    greaterThanOrEqual: 'greaterThanOrEqual',
    lessThan: 'lessThan',
    lessThanOrEqual: 'lessThanOrEqual',
};

const HeaderCellContent = (props) => {
    const {
        children,
        onFilterToggle,
    } = props;
    return (
        <Fragment>
            { children }
            <IconButton
                iconName="filter_list"
                iconSize={ ICON_SIZE.SMALL }
                onClick={ onFilterToggle } />
        </Fragment>
    );
};
HeaderCellContent.propTypes = {
    // Unused available props
    // align: PropTypes.bool,
    // column: PropTypes.object,

    children: PropTypes.oneOfType([
        PropTypes.array,
        PropTypes.node,
        PropTypes.element,
    ]),
    onFilterToggle: PropTypes.func,
};

/**
 * An extensive table grid control
 * @see https://devexpress.github.io/devextreme-reactive/react/grid/
 */
const DataTableDXG = (props) => {
    const {
        cellComponent, // Function
        columns, // [ { name: String, title?: String } ]
        contextStyles, // Object
        defaultColumnWidth, // Number
        filterDirectives, // [ { columnName: String, operation: String, value: any } ]
        filterOperations, // [ { columnName: String, operators: Array<String> } ]
        fixedColumnIndexes, // [ Number ]
        infiniteScrolling, // Boolean
        loading, // Boolean
        loadMoreData, // Function { Number } skip, { Number } take
        onColumnsReordered, // Function [ String ] array of column names in order
        onColumnsResized, // Function [ { columnName: String, width: Number } ]
        onFilterChange, // Function [ { columnName: String, operation: String, value: String } ]
        onFilterToggle, // Function { Object } event, { Object } toggle props
        onRowSelectionChange, // Function [ Number ] selected row indexes
        pageSize, // Number
        rows, // [ Object ]
        selectedRowIndexes, // [ Number ]
        showFilterRow, // Boolean
        sortDirectives, // [ { columnName: String, direction: 'asc'|'desc'|null } ]
        testID, // String
        totalRows, // Number
    } = props;

    console.log('SORT DIRECTIVES: ', sortDirectives); // eslint-disable-line
    console.log('FILTER DIRECTIVES: ', filterDirectives); // eslint-disable-line
    console.log('FILTER OPERATIONS: ', filterOperations); // eslint-disable-line

    const [
        selection,
        setSelection,
    ] = useState(selectedRowIndexes);

    useEffect(() => {
        setSelection(selectedRowIndexes);
    }, [ selectedRowIndexes ]);

    const [
        filters,
        setFilters,
    ] = useState(filterDirectives);

    const styles = getStyles(contextStyles);
    const columnOrder = columns.map((c) => (c.name));
    const columnWidths = columns.map((c) => {
        return {
            columnName: c.name,
            width: c.width || defaultColumnWidth,
        };
    });
    const fixedLeftColumns = fixedColumnIndexes.map((idx) => (columns[idx].name));

    console.log('DATA TABLE RXG: RENDER: ', props); // eslint-disable-line

    return (
        <div
            className="c-data-table__container"
            style={ styles.container }
            data-test-id={ `${testID}__data-table__container` }>
            { loading && <ActivityIndicator centered /> }
            <Grid
                columns={ columns }
                rows={ rows }>

                { /* Include Infinite Scrolling Functionality and Handling */ }
                <VirtualTableState
                    infiniteScrolling={ infiniteScrolling }
                    loading={ loading }
                    pageSize={ pageSize }
                    totalRowCount={ totalRows }
                    skip={ 0 }
                    getRows={ loadMoreData } />

                { /* Include Row Selection Handling */ }
                <SelectionState
                    selection={ selection }
                    onSelectionChange={ (...args) => {
                        setSelection(...args);
                        if (onRowSelectionChange) {
                            onRowSelectionChange(...args);
                        }
                    } } />

                { /* Include Custom Filtering Drop Down Operations */ }
                { /* showFilterRow && filterOperations.length > 0 && filterOperations.map((filterOperation) => (
                    <DataTypeProvider
                        key={ filterOperation.columnName }
                        for={ [ filterOperation.columnName ] }
                        availableFilterOperations={ filterOperation.operators } />
                )) */ }

                { /* Include Filtering Functionality and Handling */ }
                { showFilterRow &&
                    <FilteringState
                        filters={ filters }
                        onFiltersChange={ (...args) => {
                            setFilters(...args);
                            if (onFilterChange) {
                                onFilterChange(...args);
                            }
                        } } />
                }

                { /* Include Drag and Drop Functionality and Handling */ }
                <DragDropProvider />

                { /* Include Table */ }
                <VirtualTable
                    cellComponent={ cellComponent } />

                { /* Include Column Reordering Functionality and Handling */ }
                { /* Controlled With Internal Grid State */ }
                <TableColumnReordering
                    defaultOrder={ columnOrder }
                    onOrderChange={ onColumnsReordered } />

                { /* Include Column Resizing Functionality and Handling */ }
                { /* Controlled With Internal Grid State */ }
                <TableColumnResizing
                    defaultColumnWidths={ columnWidths }
                    onColumnWidthsChange={ onColumnsResized } />

                { /* Include Column Header Functionality and Handling */ }
                <TableHeaderRow
                    contentComponent={ (p) => (
                        <HeaderCellContent
                            { ...p }
                            onFilterToggle={ onFilterToggle } />
                    ) } />

                { /* Include Filter Row when enabled */ }
                { showFilterRow &&
                    <TableFilterRow
                        showFilterSelector />
                }

                { /* Include Row Selection Functionality */ }
                <TableSelection
                    selectByRowClick
                    highlightRow
                    showSelectionColumn={ false } />

                { /* Include Fixed/Frozen/Pinned columns */ }
                <TableFixedColumns
                    leftColumns={ fixedLeftColumns } />
            </Grid>
        </div>
    );
};

DataTableDXG.propTypes = {
    /** Component used to generate the table cell and its contents */
    cellComponent: PropTypes.oneOfType([
        PropTypes.func,
        PropTypes.object,
    ]),

    /** An array containing column data */
    columns: PropTypes.arrayOf(PropTypes.shape({
        /** The name identifier of the column */
        name: PropTypes.string,

        /** The column header display text */
        title: PropTypes.string,
    })).isRequired,

    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding the table */
        container: PropTypes.object,
    }),

    /** Default (px) width applied to all columns */
    defaultColumnWidth: PropTypes.number,

    /** Filter entry directives */
    filterDirectives: PropTypes.arrayOf(PropTypes.shape({
        /** The name identifier of the filter column */
        columnName: PropTypes.string,

        /** The filter operation */
        operation: PropTypes.string,

        /** The filter value */
        value: PropTypes.oneOfType([
            PropTypes.string,
            PropTypes.number,
            PropTypes.bool,
        ]),
    })),

    /** Column filter operators */
    filterOperations: PropTypes.arrayOf(PropTypes.shape({
        /** The name identifier of the filter column */
        columnName: PropTypes.string,

        /** The filter operation */
        operators: PropTypes.object,
    })),

    /** Indexes of columns to freeze and affix to the left of the table */
    fixedColumnIndexes: PropTypes.arrayOf(PropTypes.number),

    /** Enables a scrolling mode in which rows are loaded in sequence */
    infiniteScrolling: PropTypes.bool,

    /** Shows the table loading state when enabled */
    loading: PropTypes.bool,

    /**
     * Called when the viewport row threshold reaches a certain point when scrolling
     * @param {Number} skip
     * @param {Number} take
     * @returns {void}
     */
    loadMoreData: PropTypes.func,

    /**
     * Called on column reorder
     * @param {Array.<String>} columnsNamesInOrder
     * @returns {void}
     */
    onColumnsReordered: PropTypes.func,

    /**
     * Called on column resize
     * @param {Array.<{ columnName: String, width: Number }>} columnsWithWidths
     * @returns {void}
     */
    onColumnsResized: PropTypes.func,

    /**
     * @typedef {Object} Filter
     * @property {String} columnName
     * @property {FilterOperations} operation
     * @property {String} value
     *
     * Called on filter change
     * @param {Array.<Filter>} filters
     * @returns {void}
     */
    onFilterChange: PropTypes.func,

    /**
     * Called on filter toggle component click
     * @param {Object} event - the native click event
     * @param {Object} props - the clicked component props
     * @returns {void}
     */
    onFilterToggle: PropTypes.func,

    /**
     * Called on row click
     * @param {Array.<Number|String>} selectedRowIndexes
     * @returns {void}
     */
    onRowSelectionChange: PropTypes.func,

    /** Specifies the count of rows in the current virtual chunk */
    pageSize: PropTypes.number,

    /** An array containing custom data */
    rows: PropTypes.array.isRequired,

    /** Indexes of rows that show as selected in the UI */
    selectedRowIndexes: PropTypes.arrayOf(PropTypes.number),

    /** Shows the table header filter row when enabled */
    showFilterRow: PropTypes.bool,

    /** Column sorting directives */
    sortDirectives: PropTypes.arrayOf(PropTypes.shape({
        /** The name identifier of the column */
        columnName: PropTypes.string,

        /** The sorting direction reflected in the UI */
        direction: PropTypes.oneOf([
            'asc',
            'desc',
            null,
        ]),
    })),

    /** ID string used for testing */
    testID: PropTypes.string,

    /** The total number of rows to render including virtual rows when infinite scrolling is enabled */
    totalRows: PropTypes.number.isRequired,
};

DataTableDXG.defaultProps = {
    cellComponent: DataTableCell,
    contextStyles: {},
    defaultColumnWidth: DEFAULT_COLUMN_WIDTH,
    filterDirectives: [],
    filterOperations: [],
    fixedColumnIndexes: [],
    infiniteScrolling: false,
    selectedRowIndexes: [],
    sortDirectives: [],
};

export {
    DataTableCell,
    FILTER_OPERATIONS,
};

export default DataTableDXG;
