import React, { useState, useEffect } from 'react';
import * as PropTypes from 'prop-types';

import TextField, { INPUT_TYPES } from '../TextField/TextField';
import Toggle from '../Toggle/Toggle';
import DateTimePicker, { DATE_TYPES, DATE_VIEWS } from '../DateTimePicker/DateTimePicker';
import DropDown from '../DropDown/DropDown';
import IconButton, { ICON_SIZE } from '../IconButton/IconButton';

const SORT_DIRECTIONS = {
    ASC: 'ASC',
    DESC: 'DESC',
    NONE: 'NONE',
};
const FILTER_TYPES = {
    BOOLEAN: 'BOOLEAN',
    DATE: 'DATE',
    DATETIME: 'DATETIME',
    DECIMAL: 'DECIMAL',
    INTEGER: 'INTEGER',
    STRING: 'STRING',
    TIME: 'TIME',
    YEAR: 'YEAR',
};
const FILTER_OPERATORS = {
    NONE: '',
    EQUAL: 'Equal',
    NOT_EQUAL: 'Not Equal',
    GREATER_THAN: 'Greater Than',
    LESS_THAN: 'Less Than',
    CONTAINS: 'Contains',
    STARTS_WITH: 'Starts With',
    ENDS_WITH: 'Ends With',
};
const FILTER_OPTIONS = {
    [FILTER_OPERATORS.EQUAL]: {
        value: FILTER_OPERATORS.EQUAL,
        label: 'Equal',
    },
    [FILTER_OPERATORS.NOT_EQUAL]: {
        value: FILTER_OPERATORS.NOT_EQUAL,
        label: 'Not Equal',
    },
    [FILTER_OPERATORS.CONTAINS]: {
        value: FILTER_OPERATORS.CONTAINS,
        label: 'Contains',
    },
    [FILTER_OPERATORS.STARTS_WITH]: {
        value: FILTER_OPERATORS.STARTS_WITH,
        label: 'Starts With',
    },
    [FILTER_OPERATORS.ENDS_WITH]: {
        value: FILTER_OPERATORS.ENDS_WITH,
        label: 'Ends With',
    },
    [FILTER_OPERATORS.GREATER_THAN]: {
        value: FILTER_OPERATORS.GREATER_THAN,
        label: 'Greater Than',
    },
    [FILTER_OPERATORS.LESS_THAN]: {
        value: FILTER_OPERATORS.LESS_THAN,
        label: 'Less Than',
    },
};
const STRING_FILTER_OPTIONS = [
    FILTER_OPTIONS[FILTER_OPERATORS.EQUAL],
    FILTER_OPTIONS[FILTER_OPERATORS.NOT_EQUAL],
    FILTER_OPTIONS[FILTER_OPERATORS.CONTAINS],
    FILTER_OPTIONS[FILTER_OPERATORS.STARTS_WITH],
    FILTER_OPTIONS[FILTER_OPERATORS.ENDS_WITH],
];
const NUMERIC_FILTER_OPTIONS = [
    FILTER_OPTIONS[FILTER_OPERATORS.EQUAL],
    FILTER_OPTIONS[FILTER_OPERATORS.NOT_EQUAL],
    FILTER_OPTIONS[FILTER_OPERATORS.GREATER_THAN],
    FILTER_OPTIONS[FILTER_OPERATORS.LESS_THAN],
];
const extractUselessProps = (allProps, uselessProps = [ 'text' ]) => {
    const {
        [uselessProps[0]]: uselessProp,
        ...rest
    } = allProps;
    return rest;
};

const DataTableFloatingFilter = (props) => {
    console.log('DATA TABLE FLOATING FILTER: PROPS: ', props); // eslint-disable-line

    const {
        column,
        filterDefs,
        onApplyFilters,
        onChangeFilter,
        onClearAllFilters,
        onClearFilter,
        onSortRequested,
    } = props;
    const {
        colDef,
    } = column;
    const {
        field,
    } = colDef;

    // Get the current field definition
    const filterDef = filterDefs.find((def) => (def.field === field));

    // Get filter definition props
    const {
        operator,
        sortDirection,
        sortPriority,
        type,
        value: filterValue,
    } = filterDef;

    // Set initial filter field values
    const initialValue = {
        sortPriority,
        operator,
        text: (operator && FILTER_OPTIONS[operator].label) || '',
        value: filterValue,
        isAscending: sortDirection === SORT_DIRECTIONS.ASC,
        isDescending: sortDirection === SORT_DIRECTIONS.DESC,
    };
    if (!sortDirection || (sortDirection === SORT_DIRECTIONS.NONE)) {
        initialValue.isAscending = null;
        initialValue.isDescending = null;
    }

    console.log('DATA TABLE FLOATING FILTER: INITIAL VALUE: ', initialValue); // eslint-disable-line

    // Initialize filter values state
    const [
        filter,
        setFilter,
    ] = useState(initialValue);
    useEffect(() => {
        console.log('USE EFFECT'); // eslint-disable-line
    }, [ sortPriority ]);

    // Default to text input type
    let textFieldType = INPUT_TYPES.TEXT;
    let stringCriteria = STRING_FILTER_OPTIONS;

    // Default to DateTime date input type, format and views
    let dateFieldType = DATE_TYPES.DATETIME;
    let dateFieldFormat = 'M/dd/yy, h:mm a';
    let dateTimeViews = [
        DATE_VIEWS.DATE,
        DATE_VIEWS.HOURS,
        DATE_VIEWS.MINUTES,
        DATE_VIEWS.YEAR,
    ];
    let FieldFilterComponent = null;

    switch (type) {
        case FILTER_TYPES.BOOLEAN:
            FieldFilterComponent = (
                <Toggle
                    checked={ filter.value || false }
                    onChange={ (event, props) => {
                        const {
                            currentTarget,
                        } = event;
                        const {
                            checked,
                        } = currentTarget;
                        const newFilter = {
                            ...filter,
                            value: checked,
                        };
                        setFilter(newFilter);
                        if (onChangeFilter) {
                            onChangeFilter(event, props, field, extractUselessProps(newFilter, [ 'text' ]));
                        }
                    } } />
            );
            break;

        case FILTER_TYPES.DATE:
            if (type === FILTER_TYPES.DATE) {
                dateFieldType = DATE_TYPES.DATE;
                dateFieldFormat = 'M/dd/yy';
                dateTimeViews = [
                    DATE_VIEWS.DATE,
                    DATE_VIEWS.YEAR,
                ];
            }
        // eslint-disable-next-line no-fallthrough
        case FILTER_TYPES.YEAR:
            if (type === FILTER_TYPES.YEAR) {
                dateFieldType = DATE_TYPES.DATE;
                dateFieldFormat = 'yyyy';
                dateTimeViews = [ DATE_VIEWS.YEAR ];
            }
        // eslint-disable-next-line no-fallthrough
        case FILTER_TYPES.TIME:
            if (type === FILTER_TYPES.TIME) {
                dateFieldType = DATE_TYPES.TIME;
                dateFieldFormat = 'h:mm a';
                dateTimeViews = [
                    DATE_VIEWS.HOURS,
                    DATE_VIEWS.MINUTES,
                ];
            }
        // eslint-disable-next-line no-fallthrough
        case FILTER_TYPES.DATETIME:
            FieldFilterComponent = (
                <div style={ {
                    display: 'flex',
                    flexDirection: 'column',
                } }>
                    <DropDown
                        items={ NUMERIC_FILTER_OPTIONS }
                        onChange={ (event, props) => {
                            const {
                                target,
                            } = event;
                            const {
                                value,
                            } = target;
                            const {
                                text, // eslint-disable-line
                            } = props;
                            const newFilter = {
                                ...filter,
                                operator: value,
                                text,
                            };
                            setFilter(newFilter);
                            if (onChangeFilter) {
                                onChangeFilter(event, props, field, extractUselessProps(newFilter, [ 'text' ]));
                            }
                        } }
                        renderValue={ (value) => (value || 'Filter type...') }
                        renderItem={ (MenuItem, item) => (
                            <MenuItem
                                value={ item.value }
                                text={ item.label }
                                key={ item.value } />
                        ) }
                        value={ filter.text || '' } />
                    <DateTimePicker
                        emptyLabel="Filter date..."
                        selectedDate={ filter.value || null }
                        onChange={ (value) => {
                            const newFilter = {
                                ...filter,
                                value,
                            };
                            setFilter(newFilter);
                            if (onChangeFilter) {
                                onChangeFilter(null, props, field, extractUselessProps(newFilter, [ 'text' ]));
                            }
                        } }
                        format={ dateFieldFormat }
                        mode={ dateFieldType }
                        views={ dateTimeViews } />
                </div>
            );
            break;

        case FILTER_TYPES.DECIMAL:
            if (type === FILTER_TYPES.DECIMAL) {
                textFieldType = INPUT_TYPES.NUMBER;
                stringCriteria = NUMERIC_FILTER_OPTIONS;
            }
        // eslint-disable-next-line no-fallthrough
        case FILTER_TYPES.INTEGER:
            if (type === FILTER_TYPES.INTEGER) {
                textFieldType = INPUT_TYPES.NUMBER;
                stringCriteria = NUMERIC_FILTER_OPTIONS;
            }
        // eslint-disable-next-line no-fallthrough
        case FILTER_TYPES.STRING:
            FieldFilterComponent = (
                <div style={ {
                    display: 'flex',
                    flexDirection: 'column',
                } }>
                    <DropDown
                        items={ stringCriteria }
                        onChange={ (event, props) => {
                            const {
                                target,
                            } = event;
                            const {
                                value,
                            } = target;
                            const {
                                text, // eslint-disable-line
                            } = props;
                            const newFilter = {
                                ...filter,
                                operator: value,
                                text,
                            };
                            setFilter(newFilter);
                            if (onChangeFilter) {
                                onChangeFilter(event, props, field, extractUselessProps(newFilter, [ 'text' ]));
                            }
                        } }
                        renderValue={ (value) => (value || 'Filter type...') }
                        renderItem={ (MenuItem, item) => (
                            <MenuItem
                                value={ item.value }
                                text={ item.label }
                                key={ item.value } />
                        ) }
                        value={ filter.text || '' } />
                    <TextField
                        placeholder="Filter text..."
                        type={ textFieldType }
                        onChange={ (event) => {
                            const {
                                currentTarget,
                            } = event;
                            const {
                                value,
                            } = currentTarget;
                            const newFilter = {
                                ...filter,
                                value,
                            };
                            setFilter(newFilter);
                            if (onChangeFilter) {
                                onChangeFilter(event, props, field, extractUselessProps(newFilter, [ 'text' ]));
                            }
                        } }
                        value={ (filter.value && filter.value.toString()) || '' } />
                </div>
            );
            break;

        default: {
            console.warn('NO COMPONENT TYPE'); // eslint-disable-line
            FieldFilterComponent = (
                <div>EMPTY</div>
            );
        }
    }

    // Generate sort ascending button props
    const sortAscendingProps = {
        contextStyles: {
            icon: {
                // Invert icon for sort descending as one was not in the core library
                transform: 'scaleY(-1)',
            },
        },
        badgeText: (filter.isAscending && sortPriority) ? sortPriority.toString() : '',
        iconName: 'sort',
        iconSize: ICON_SIZE.SMALL,
        onClick: (event, props) => {
            console.log('DATA TABLE: FILTER: SORT ASCENDING EVENT: ', field, event, props); // eslint-disable-line
            const newFilter = {
                ...filter,
                isAscending: true,
                isDescending: false,
            };
            setFilter(newFilter);
            if (onSortRequested) {
                const filterProps = extractUselessProps(newFilter, [ 'text' ]);
                console.log('DATA TABLE: FILTER: SORT ASCENDING: ON_SORT_REQUESTED: ', event, props, field, SORT_DIRECTIONS.ASC, filterProps); // eslint-disable-line
                onSortRequested(event, props, field, SORT_DIRECTIONS.ASC, filterProps);
            }
        },
        title: 'sort ascending',
        disabled: filter.isAscending,
    };

    // Generate sort descending button props
    const sortDescendingProps = {
        badgeText: (filter.isDescending && sortPriority) ? sortPriority.toString() : '',
        iconName: 'sort',
        iconSize: ICON_SIZE.SMALL,
        onClick: (event, props) => {
            console.log('DATA TABLE: FILTER: SORT DESCENDING EVENT: ', field, event, props); // eslint-disable-line
            const newFilter = {
                ...filter,
                isAscending: false,
                isDescending: true,
            };
            setFilter(newFilter);
            if (onSortRequested) {
                const filterProps = extractUselessProps(newFilter, [ 'text' ]);
                console.log('DATA TABLE: FILTER: SORT DESCENDING: ON_SORT_REQUESTED: ', event, props, field, SORT_DIRECTIONS.DESC, filterProps); // eslint-disable-line
                onSortRequested(event, props, field, SORT_DIRECTIONS.DESC, filterProps);
            }
        },
        title: 'sort descending',
        disabled: filter.isDescending,
    };

    // Generate clear sort button props
    const clearSortProps = {
        iconName: 'clear',
        iconSize: ICON_SIZE.SMALL,
        onClick: (event, props) => {
            console.log('DATA TABLE: FILTER: SORT CLEAR EVENT: ', field, event, props); // eslint-disable-line
            const newFilter = {
                ...filter,
                isAscending: null,
                isDescending: null,
            };
            setFilter(newFilter);
            if (onSortRequested) {
                const filterProps = extractUselessProps(newFilter, [ 'text' ]);
                console.log('DATA TABLE: FILTER: CLEAR SORT: ON_SORT_REQUESTED: ', event, props, field, SORT_DIRECTIONS.NONE, filterProps); // eslint-disable-line
                onSortRequested(event, props, field, SORT_DIRECTIONS.NONE, filterProps);
            }
        },
        title: 'clear sort',
        disabled: filter.isAscending === null && filter.isDescending === null,
    };

    // Generate apply filters button props
    const applyFiltersProps = {
        iconName: 'done',
        iconSize: ICON_SIZE.SMALL,
        onClick: (event, props) => {
            console.log('DATA TABLE: FILTER: APPLY FILTERS EVENT: ', field, event, props); // eslint-disable-line
            if (onApplyFilters) {
                const filterProps = extractUselessProps(filter, [ 'text' ]);
                onApplyFilters(event, props, field, filterProps);
            }
        },
        title: 'apply filters',
    };

    // Generate clear filter button props
    const clearFilterProps = {
        iconName: 'undo',
        iconSize: ICON_SIZE.SMALL,
        onClick: (event, props) => {
            console.log('DATA TABLE: FILTER: CLEAR FILTER EVENT: ', field, event, props); // eslint-disable-line
            if (onClearFilter) {
                const filterProps = extractUselessProps(filter, [ 'text' ]);
                onClearFilter(event, props, field, filterProps);
            }
        },
        title: 'reset filter',
    };

    // Generate clear all filters button props
    const clearAllFiltersProps = {
        iconName: 'block',
        iconSize: ICON_SIZE.SMALL,
        onClick: (event, props) => {
            console.log('DATA TABLE: FILTER: CLEAR ALL FILTERS EVENT: ', field, event, props); // eslint-disable-line
            if (onClearAllFilters) {
                const filterProps = extractUselessProps(filter, [ 'text' ]);
                onClearAllFilters(event, props, field, filterProps);
            }
        },
        title: 'reset all filters',
    };

    return (
        <div style={ {
            display: 'flex',
            flexDirection: 'column',
        } }>
            <div style={ {
                display: 'flex',
                flexDirection: 'row',
            } }>
                <IconButton { ...sortAscendingProps } />
                <IconButton { ...sortDescendingProps } />
                <IconButton { ...clearSortProps } />
            </div>

            { FieldFilterComponent }

            <div style={ {
                display: 'flex',
                flexDirection: 'row',
            } }>
                <IconButton { ...applyFiltersProps } />
                <IconButton { ...clearFilterProps } />
                <IconButton { ...clearAllFiltersProps } />
            </div>
        </div>
    );
};

DataTableFloatingFilter.propTypes = {
    /**
     * @typedef {Object} FilterData
     * @property {Boolean} isAscending
     * @property {Boolean} isDescending
     * @property {String} operator
     * @property {Boolean|Number|String|Date} value
     */

    /** ag-grid-react column data */
    column: PropTypes.object,

    /** Array containing filter definition objects */
    filterDefs: PropTypes.arrayOf(PropTypes.shape({
        /** Unique column id which maps to row property */
        field: PropTypes.string,

        /** Filter field operator value */
        operator: PropTypes.oneOf([
            FILTER_OPERATORS.NONE,
            FILTER_OPERATORS.EQUAL,
            FILTER_OPERATORS.NOT_EQUAL,
            FILTER_OPERATORS.GREATER_THAN,
            FILTER_OPERATORS.LESS_THAN,
            FILTER_OPERATORS.CONTAINS,
            FILTER_OPERATORS.STARTS_WITH,
            FILTER_OPERATORS.ENDS_WITH,
        ]),

        /** Sort direction */
        sortDirection: PropTypes.oneOf([
            SORT_DIRECTIONS.NONE,
            SORT_DIRECTIONS.ASC,
            SORT_DIRECTIONS.DESC,
        ]),

        /** Sort priority */
        sortPriority: PropTypes.number,

        /** Filter field input type */
        type: PropTypes.oneOf([
            FILTER_TYPES.BOOLEAN,
            FILTER_TYPES.DATE,
            FILTER_TYPES.DATETIME,
            FILTER_TYPES.DECIMAL,
            FILTER_TYPES.INTEGER,
            FILTER_TYPES.STRING,
            FILTER_TYPES.TIME,
            FILTER_TYPES.YEAR,
        ]),

        /** Filter field input value */
        value: PropTypes.oneOfType([
            PropTypes.bool,
            PropTypes.number,
            PropTypes.string,
            PropTypes.instanceOf(Date),
        ]),
    })).isRequired,

    /**
     * Called on click of apply filters button
     * @param {Object} event
     * @param {Object} props
     * @param {String} field
     * @param {FilterData} filterValues
     */
    onApplyFilters: PropTypes.func,

    /**
     * Called after change of a filter value
     * @param {Object} event
     * @param {Object} props
     * @param {String} field
     * @param {FilterData} filterValues
     */
    onChangeFilter: PropTypes.func,

    /**
     * Called on click of clear all filters button
     * @param {Object} event
     * @param {Object} props
     * @param {String} field
     * @param {FilterData} filterValues
     */
    onClearAllFilters: PropTypes.func,

    /**
     * Called on click of clear single filter button
     * @param {Object} event
     * @param {Object} props
     * @param {String} field
     * @param {FilterData} filterValues
     */
    onClearFilter: PropTypes.func,

    /**
     * Called after change of a filter value
     * @param {Object} event
     * @param {Object} props
     * @param {String} field
     * @param {String} sortOrder
     * @param {FilterData} filterValues
     */
    onSortRequested: PropTypes.func,
};

DataTableFloatingFilter.defaultProps = {};

export default DataTableFloatingFilter;
export {
    FILTER_TYPES,
    FILTER_OPERATORS,
    SORT_DIRECTIONS,
    FILTER_OPTIONS,
    STRING_FILTER_OPTIONS,
    NUMERIC_FILTER_OPTIONS,
};
